# Matière Formule v 1.3 - César Vonc - http://code.vonc.fr/

import os
import math
import random
import c4d
from c4d import plugins, Vector, utils
from c4d.utils.noise import Noise
from math import *

MODULE_ID = 1028230

MATFORMULE_DEFCODE = 1000
MATFORMULE_NOM = 1001

MATFORMULE_SOURCE = 1000
MATFORMULE_ECHELLE = 1001
MATFORMULE_CONTR = 1002
MATFORMULE_ECHXYZ = 1003
MATFORMULE_COL1 = 1004
MATFORMULE_COL2 = 1005
MATFORMULE_LUM = 1006
MATFORMULE_CODE = 1007
MATFORMULE_COL3 = 1008
MATFORMULE_ACT = 1009
MATFORMULE_MODE = 1010
MATFORMULE_MODE_UV = 0
MATFORMULE_MODE_TEX = 3
MATFORMULE_MODE_OBJ = 2
MATFORMULE_MODE_GLO = 1
MATFORMULE_MODE_CAM = 4
MATFORMULE_MODE_ECR = 5
MATFORMULE_MIN = 1011
MATFORMULE_MAX = 1012
MATFORMULE_INFO = 1013
MATFORMULE_PROF = 1014
MATFORMULE_PROF_SANS = 0
MATFORMULE_PROF_LIN_SRGB = 1
MATFORMULE_PROF_SRGB_LIN = 2
MATFORMULE_PROF_LIN_AFF = 3
MATFORMULE_PROF_SRGB_AFF = 4
MATFORMULE_APE = 1015
MATFORMULE_INTROG = 1016
MATFORMULE_INTRO = 1017

class MatFormule(plugins.ShaderData):
	def __init__(self) :
		self.src = 0.0
		self.ech = 1.0
		self.echxyz = Vector(1.0)
		self.contra = 0.0
		self.lumi = 0.0
		self.cola = Vector(0)
		self.colb = Vector(1.0)
		self.col_ = True
		self.con_ = True
		self.colc = Vector(1.0, 0.0, 0.0)
		self.intro = ('#import os\n#import math\n#import random\n#import c4d\n#from c4d import plugins, Vector, utils\n#from c4d.utils.noise import Noise\n#from math import *\n' + 
						plugins.GeLoadString(MATFORMULE_DEFCODE).replace('#', '\n#'))
		self.codetxt = 'n = Noise(xyz, (src+cos(x)+cos(y)+cos(z))*10)\nr = v = b = n'
		self.code = compile(self.codetxt, '<string>', 'exec')
		self.mode = 1
		self.min = True
		self.max = False
		self.prof = 2
		self.irs = None
		self.ape = True
	
	def Init(self, node):
		donnees = node.GetDataInstance()
		donnees.SetReal(MATFORMULE_SOURCE, self.src)
		donnees.SetReal(MATFORMULE_ECHELLE, self.ech)
		donnees.SetVector(MATFORMULE_ECHXYZ, self.echxyz)
		donnees.SetReal(MATFORMULE_CONTR, self.contra)
		donnees.SetReal(MATFORMULE_LUM, self.lumi)
		donnees.SetVector(MATFORMULE_COL1, self.cola)
		donnees.SetVector(MATFORMULE_COL2, self.colb)
		donnees.SetString(MATFORMULE_CODE, self.codetxt)
		donnees.SetVector(MATFORMULE_COL3, self.colc)
		donnees.SetLong(MATFORMULE_MODE, self.mode)
		donnees.SetBool(MATFORMULE_MIN, self.min)
		donnees.SetBool(MATFORMULE_MAX, self.max)
		donnees.SetLong(MATFORMULE_PROF, self.prof)
		donnees.SetBool(MATFORMULE_APE, self.ape)
		donnees.SetString(MATFORMULE_INTRO, self.intro)
		return True
	
	def recup(self, sh):
		donnees = sh.GetDataInstance()
		donnees.SetString(MATFORMULE_INTRO, self.intro)
		self.src = donnees.GetReal(MATFORMULE_SOURCE)
		self.ech = donnees.GetReal(MATFORMULE_ECHELLE)
		self.echxyz = donnees.GetVector(MATFORMULE_ECHXYZ)
		self.contra = donnees.GetReal(MATFORMULE_CONTR)
		self.lumi = donnees.GetReal(MATFORMULE_LUM)
		if self.contra == 0 and self.lumi == 0:
			self.con_ = False
		else: self.con_ = True
		self.cola = donnees.GetVector(MATFORMULE_COL1)
		self.colb = donnees.GetVector(MATFORMULE_COL2)
		if self.colb == Vector(1.0) and self.cola == Vector():
			self.col_ = False
		else: self.col_ = True
		txt = donnees.GetString(MATFORMULE_CODE)
		self.colc = donnees.GetVector(MATFORMULE_COL3)
		self.SetExceptionColor(self.colc)
		self.codetxt = txt
		self.code = compile(txt, '<string>', 'exec')
		self.mode = donnees.GetLong(MATFORMULE_MODE)
		self.min = donnees.GetBool(MATFORMULE_MIN)
		self.max = donnees.GetBool(MATFORMULE_MAX)
		self.prof = donnees.GetLong(MATFORMULE_PROF)
		self.ape = donnees.GetBool(MATFORMULE_APE)
		random.seed(abs(self.src) + 1)
	
	def InitRender(self, sh, irs):
		self.recup(sh)
		self.irs = irs
		return 0
	
	def Output(self, sh, cd):
		__vec = Vector()
		
		if self.mode == MATFORMULE_MODE_GLO : # Espace Monde
			if cd.vd : # 3D
				__vec = cd.vd.p
			else : # Aperçu
				if self.ape : __vec = (cd.p - 0.5) * 100.0
				else : return __vec
		
		elif self.mode == MATFORMULE_MODE_UV : # Espace UV
			__vec = cd.p
		
		elif self.mode == MATFORMULE_MODE_OBJ : # Espace Objet
			if cd.vd : # 3D
				__vec = cd.vd.p
			else : # Aperçu
				if self.ape : __vec = (cd.p - 0.5) * 100.0
				else : return __vec
			
		__vec *= self.ech
		
		xyz = __vec.__rxor__(self.echxyz)
		x = xyz.x
		y = xyz.y
		z = xyz.z
		src = self.src
		rvb = None
		r = v = b = 0.0
		irs = self.irs
		
		exec self.code
		
		if (rvb is None) : __vec = Vector(r, v, b)
		else : __vec = rvb
		
		if self.col_ :
			dif = self.colb - self.cola
			__vec = __vec.__rxor__(dif) + self.cola
		if self.con_ :
			__vec = ((__vec-0.5) * (self.contra + 1.0)) + 0.5 + self.lumi
		if self.min :
			if __vec.x < 0 : __vec.x = 0.0
			if __vec.y < 0 : __vec.y = 0.0
			if __vec.z < 0 : __vec.z = 0.0
		if self.max :
			if __vec.x > 1 : __vec.x = 1.0
			if __vec.y > 1 : __vec.y = 1.0
			if __vec.z > 1 : __vec.z = 1.0
		__vec = utils.TransformColor(__vec, self.prof)
		return __vec
	
	def FreeRender(self, sh): # Libère la mémoire
		return

if __name__=='__main__':
	plugins.RegisterShaderPlugin(MODULE_ID, plugins.GeLoadString(MATFORMULE_NOM), 0, MatFormule, "matformule", 0)
